# -*- coding: utf-8 -*-

"""
DevEmailBackend
===============

Questo Backend permette di intercettare tutte le email inviate con send_email
e sovrascrivere il valore di alcuni campi applicando un set di configurazioni.

Quando è utile
----------------

in ambienti di staging per evitare di mandare email al cliente finale dirottandole ad un indirizzo di test


Esempio di utilizzo
-------------------

::

  EMAIL_SUBJECT_PREFIX = '[DEV:OFFICE2020.PROJECT.IT]' # Già presente nei vari progetti

  EMAIL_BACKEND = 'thx.core.backends.DevEmailBackend'
  DEV_EMAIL_RECIPIENT = ['code@thux.it', ]
  DEV_EMAIL_BCC = ['code@thux.it', ]
  DEV_EMAIL_CC = ['code@thux.it', ]

Classi
========

.. autoclass:: DevEmailBackend
   :members:

"""
from django.conf import settings
from django.core.mail.backends.smtp import EmailBackend


class DevEmailBackend(EmailBackend):
    """
    Questo Backend permette di intercettare tutte le email inviate con send_email
    e sovrascrivere il valore di alcuni campi con la seguente configurazione:

    * email_message.to viene sostituito con settings.DEV_EMAIL_RECIPIENT
    * email_message.bcc viene sostituito con settings.DEV_EMAIL_BCC
    * email_message.cc viene sostituito con settings.DEV_EMAIL_CC
    * email_message.subject viene preceduto dal settings settings.EMAIL_SUBJECT_PREFIX
    """
    def _send(self, email_message):
        email_message.to = settings.DEV_EMAIL_RECIPIENT
        if email_message.bcc:
            email_message.bcc = settings.DEV_EMAIL_BCC
        if email_message.cc:
            email_message.cc = settings.DEV_EMAIL_CC
        email_message.subject = f'{settings.EMAIL_SUBJECT_PREFIX} {email_message.subject}'
        super()._send(email_message)
